--
-- exclusão de chaves estrangeiras
--

ALTER TABLE IF EXISTS disciplina
   DROP CONSTRAINT 
   IF EXISTS fk_disciplina_depto;

ALTER TABLE IF EXISTS periodo
   DROP CONSTRAINT 
   IF EXISTS fk_periodo_sala;

ALTER TABLE IF EXISTS periodo
   DROP CONSTRAINT 
   IF EXISTS fk_periodo_oferta;

ALTER TABLE IF EXISTS prereq
   DROP CONSTRAINT 
   IF EXISTS fk_prereq_eh_pre_discipli;

ALTER TABLE IF EXISTS prereq
   DROP CONSTRAINT 
   IF EXISTS fk_prereq_tem_pre_discipli;

ALTER TABLE IF EXISTS professor
   DROP CONSTRAINT 
   IF EXISTS fk_professor_titulacao;

ALTER TABLE IF EXISTS professor
   DROP CONSTRAINT 
   IF EXISTS fk_professor_depto;

ALTER TABLE IF EXISTS prof_oferta
   DROP CONSTRAINT 
   IF EXISTS 
      fk_prof_oferta_professor;

ALTER TABLE IF EXISTS prof_oferta
   DROP CONSTRAINT 
   IF EXISTS fk_prof_oferta_oferta;

ALTER TABLE IF EXISTS sala
   DROP CONSTRAINT 
   IF EXISTS fk_sala_predio;

ALTER TABLE IF EXISTS oferta
   DROP CONSTRAINT 
   IF EXISTS fk_oferta_disciplina;

--
-- exclusão de tabelas
--

DROP TABLE IF EXISTS depto;
DROP TABLE IF EXISTS disciplina;
DROP TABLE IF EXISTS periodo;
DROP TABLE IF EXISTS predio;
DROP TABLE IF EXISTS prereq;
DROP TABLE IF EXISTS professor;
DROP TABLE IF EXISTS prof_oferta;
DROP TABLE IF EXISTS sala;
DROP TABLE IF EXISTS titulacao;
DROP TABLE IF EXISTS oferta;

--
-- criação de tabelas
--

-- tabela depto -----------------

CREATE TABLE depto (
   cod_depto  CHAR(5)      NOT NULL,
   nome_depto VARCHAR(40)  NOT NULL,
   CONSTRAINT pk_depto 
      PRIMARY KEY (cod_depto)
);

-- tabela disciplina -----------------

CREATE TABLE disciplina (
   cod_depto     CHAR(5)     NOT NULL,
   num_disc      INTEGER     NOT NULL,
   nome_disc     VARCHAR(40) NOT NULL,
   creditos_disc INTEGER     NOT NULL,
   CONSTRAINT pk_disciplina 
      PRIMARY KEY (cod_depto, num_disc)
);

-- tabela periodo --------------
  
CREATE TABLE periodo (
   ano_sem     INTEGER NOT NULL,
   cod_depto   CHAR(5) NOT NULL,
   num_disc    INTEGER NOT NULL,
   sigla_ofer  CHAR(2) NOT NULL,
   dia_sem     INTEGER NOT NULL,
   hora_inicio INTEGER NOT NULL,
   cod_pred    INTEGER NULL,
   num_sala    INTEGER NULL,
   num_horas   INTEGER NOT NULL,
   CONSTRAINT pk_periodo 
      PRIMARY KEY (
         cod_depto, 
         num_disc, 
         ano_sem, 
         sigla_ofer, 
         dia_sem, 
         hora_inicio
      )
);

-- tabela predio -----------------
CREATE TABLE predio (
   cod_pred  INTEGER     NOT NULL,
   nome_pred VARCHAR(40) NOT NULL,
   CONSTRAINT pk_predio 
      PRIMARY KEY (cod_pred)
);
-- tabela prereq -----------------

CREATE TABLE prereq (
   cod_depto        CHAR(5) NOT NULL,
   num_disc         INTEGER NOT NULL,
   cod_depto_prereq CHAR(5) NOT NULL,
   num_disc_prereq  INTEGER NOT NULL,
   CONSTRAINT pk_prereq 
      PRIMARY KEY (
         cod_depto_prereq, 
         cod_depto, 
         num_disc, 
         num_disc_prereq
      )
);

-- tabela professor --------------

CREATE TABLE professor (
   cod_prof  INTEGER     NOT NULL,
   nome_prof VARCHAR(40) NULL,
   cod_depto CHAR(5)     NOT NULL,
   cod_tit   INTEGER     NULL,
   CONSTRAINT pk_professor 
      PRIMARY KEY (cod_prof)
);

-- tabela prof_oferta ---------

CREATE TABLE prof_oferta (
   ano_sem    INTEGER NOT NULL,
   cod_depto  CHAR(5) NOT NULL,
   num_disc   INTEGER NOT NULL,
   sigla_ofer CHAR(2) NOT NULL,
   cod_prof   INTEGER NOT NULL,
   CONSTRAINT pk_prof_oferta 
      PRIMARY KEY (
         cod_depto, 
         num_disc, 
         ano_sem, 
         sigla_ofer, 
         cod_prof
      )
);

-- tabela sala -------------------------
CREATE TABLE sala (
   cod_pred        INTEGER     NOT NULL,
   num_sala        INTEGER     NOT NULL,
   descricao_sala  VARCHAR(40) NULL,
   capacidade_sala INTEGER     NULL,
   CONSTRAINT pk_sala 
      PRIMARY KEY (
         cod_pred, 
         num_sala
      )
);
-- tabela titulacao -------------

CREATE TABLE titulacao (
   cod_tit  INTEGER     NOT NULL,
   nome_tit VARCHAR(40) NOT NULL,
   CONSTRAINT pk_titulacao 
      PRIMARY KEY (cod_tit)
);

-- tabela oferta -------------

CREATE TABLE oferta (
   ano_sem         INTEGER NOT NULL,
   cod_depto       CHAR(5) NOT NULL,
   num_disc        INTEGER NOT NULL,
   sigla_ofer      CHAR(2) NOT NULL,
   capacidade_ofer INTEGER NULL,
   CONSTRAINT pk_oferta 
      PRIMARY KEY (
         cod_depto, 
         num_disc, 
         ano_sem, 
         sigla_ofer
      )
);

--
-- adição de chaves estrangeiras
--

ALTER TABLE IF EXISTS disciplina
   ADD CONSTRAINT 
      fk_disciplina_depto 
   FOREIGN KEY (cod_depto)
      REFERENCES depto (cod_depto)
         ON DELETE RESTRICT 
         ON UPDATE CASCADE;

ALTER TABLE IF EXISTS periodo
   ADD CONSTRAINT 
      fk_periodo_sala 
   FOREIGN KEY (cod_pred, num_sala)
      REFERENCES sala (cod_pred, num_sala)
         ON DELETE RESTRICT 
         ON UPDATE CASCADE;

ALTER TABLE IF EXISTS periodo
   ADD CONSTRAINT 
      fk_periodo_oferta 
   FOREIGN KEY (cod_depto, num_disc, 
                ano_sem, sigla_ofer
               )
      REFERENCES oferta (
         cod_depto, num_disc, 
         ano_sem, sigla_ofer
        )
         ON DELETE CASCADE 
         ON UPDATE CASCADE;
         

ALTER TABLE IF EXISTS prereq
   ADD CONSTRAINT 
      fk_prereq_eh_pre_discipli 
   FOREIGN KEY (cod_depto_prereq, 
                num_disc_prereq
               )
      REFERENCES disciplina 
            (cod_depto, num_disc)
         ON DELETE RESTRICT 
         ON UPDATE CASCADE;

ALTER TABLE IF EXISTS prereq
   ADD CONSTRAINT 
      fk_prereq_tem_pre_discipli 
   FOREIGN KEY (cod_depto, num_disc)
      REFERENCES disciplina (cod_depto, 
                             num_disc
                            )
         ON DELETE CASCADE 
         ON UPDATE CASCADE;


ALTER TABLE IF EXISTS professor
   ADD CONSTRAINT 
      fk_professor_titulacao 
   FOREIGN KEY (cod_tit)
      REFERENCES titulacao (cod_tit)
         ON DELETE SET NULL 
         ON UPDATE CASCADE;


ALTER TABLE IF EXISTS professor
   ADD CONSTRAINT 
      fk_professor_depto 
   FOREIGN KEY (cod_depto)
      REFERENCES depto (cod_depto)
         ON DELETE RESTRICT 
         ON UPDATE CASCADE;
         

ALTER TABLE IF EXISTS prof_oferta
   ADD CONSTRAINT 
      fk_prof_oferta_professor 
   FOREIGN KEY (cod_prof)
      REFERENCES professor (cod_prof)
         ON DELETE RESTRICT 
         ON UPDATE CASCADE;


ALTER TABLE IF EXISTS prof_oferta
   ADD CONSTRAINT 
      fk_prof_oferta_oferta 
   FOREIGN KEY (cod_depto, num_disc, 
                ano_sem, sigla_ofer
               )
      REFERENCES oferta 
         (cod_depto, num_disc, 
          ano_sem, sigla_ofer
         )
         ON DELETE CASCADE 
         ON UPDATE CASCADE;


ALTER TABLE IF EXISTS sala
   ADD CONSTRAINT 
      fk_sala_predio 
   FOREIGN KEY (cod_pred)
      REFERENCES predio (cod_pred)
         ON DELETE RESTRICT  
         ON UPDATE CASCADE;
ALTER TABLE IF EXISTS oferta
   ADD CONSTRAINT 
      fk_oferta_disciplina 
   FOREIGN KEY (cod_depto, num_disc)
      REFERENCES disciplina 
           (cod_depto, num_disc)
         ON DELETE RESTRICT 
         ON UPDATE CASCADE;